@@ -21,6 +21,7 @@ from .task import Task, BulkTaskResponse
 from .pool import Pool
 from .bucket import Bucket
 from .job import Job
+from ._filter import all_tag_filter
 from .exceptions import (QarnotGenericException, BucketStorageUnavailableException, UnauthorizedException,
                          MissingTaskException, MissingPoolException, MissingJobException)
 import requests
@@ -48,7 +49,7 @@ class Connection(object):
     def __init__(self, fileconf=None, client_token=None,
                  cluster_url=None, cluster_unsafe=False, cluster_timeout=None,
                  storage_url=None, storage_unsafe=False,
-                 retry_count=5, retry_wait=1.0):
+                 retry_count=5, retry_wait=1.0, cluster_custom_certificate=None, storage_custom_certificate=None):
         """Create a connection to a cluster with given config file, options or environment variables.
         Available environment variable are
         `QARNOT_CLUSTER_URL`, `QARNOT_CLUSTER_UNSAFE`, `QARNOT_CLUSTER_TIMEOUT` and `QARNOT_CLIENT_TOKEN`.
@@ -96,6 +97,8 @@ class Connection(object):
                 self.timeout = fileconf.get('cluster_timeout')
                 if fileconf.get('cluster_unsafe'):
                     self._http.verify = False
+                elif fileconf.get('cluster_custom_certificate'):
+                    self._http.verify = fileconf.get('cluster_custom_certificate')
             else:
                 cfg = config.ConfigParser()
                 with open(fileconf) as cfgfile:
@@ -119,13 +122,19 @@ class Connection(object):
                     if cfg.has_option('cluster', 'unsafe') \
                        and cfg.getboolean('cluster', 'unsafe'):
                         self._http.verify = False
+                    elif cfg.has_option('cluster', 'custom_certificate'):
+                        self._http.verify = cfg.get('cluster', 'custom_certificate')
                     if cfg.has_option('storage', 'unsafe') \
                        and cfg.getboolean('storage', 'unsafe'):
                         storage_unsafe = True
+                    if cfg.has_option('storage', 'custom_certificate'):
+                        storage_custom_certificate = cfg.get('storage', 'custom_certificate')
         else:
             self.cluster = cluster_url
             self.timeout = cluster_timeout
             self._http.verify = not cluster_unsafe
+            if not cluster_unsafe and cluster_custom_certificate:
+                self._http.verify = cluster_custom_certificate
             self.storage = storage_url
             auth = client_token
 
@@ -170,16 +179,22 @@ class Connection(object):
         session = boto3.session.Session()
         conf = botocore.config.Config(user_agent=self._version)
 
+        should_verify_or_certificate_path = True
+        if storage_unsafe:
+            should_verify_or_certificate_path = not storage_unsafe
+        elif storage_custom_certificate is not None:
+            should_verify_or_certificate_path = storage_custom_certificate
+
         self._s3client = session.client(service_name='s3',
                                         aws_access_key_id=user.email,
                                         aws_secret_access_key=auth,
-                                        verify=(not storage_unsafe),
+                                        verify=should_verify_or_certificate_path,
                                         endpoint_url=self.storage,
                                         config=conf)
         self._s3resource = session.resource(service_name='s3',
                                             aws_access_key_id=user.email,
                                             aws_secret_access_key=auth,
-                                            verify=(not storage_unsafe),
+                                            verify=should_verify_or_certificate_path,
                                             endpoint_url=self.storage,
                                             config=conf)
 
@@ -425,9 +440,13 @@ class Connection(object):
         buckets = [Bucket(self, x.name, create=False) for x in self._s3resource.buckets.all()]
         return buckets
 
-    def pools(self, summary=True):
+    def pools(self, summary=True, tags_intersect=None):
         """Get the list of pools stored on this cluster for this user.
 
+        :param bool summary: only get the summaries.
+        :param tags_intersect: Desired filtering tags, all of them
+        :type tags_intersect: list of :class:`str`, optional
+
         :rtype: List of :class:`~qarnot.pool.Pool`.
         :returns: Pools stored on the cluster owned by the user.
 
@@ -435,14 +454,24 @@ class Connection(object):
         :raises qarnot.exceptions.QarnotGenericException: API general error, see message for details
         """
         url = get_url('pools summaries') if summary else get_url('pools')
-        response = self._get(url)
+
+        if tags_intersect:
+            tag_filter = all_tag_filter(tags_intersect)
+            response = self._post(get_url('pools search'), tag_filter)
+        else:
+            response = self._get(url)
         raise_on_error(response)
         return [Pool.from_json(self, pool, summary) for pool in response.json()]
 
-    def tasks(self, tags=None, summary=True):
+    def tasks(self, tags=None, summary=True, tags_intersect=None):
         """Get the list of tasks stored on this cluster for this user.
 
-        :param List of :class:`str` tags: Desired filtering tags
+        :param tags: Desired filtering tags, any of them
+        :type tags: list of :class:`str`, optional
+        :param bool summary: only get the summaries.
+        :param tags_intersect: Desired filtering tags, all of them
+        :type tags_intersect: list of :class:`str`, optional
+
         :rtype: List of :class:`~qarnot.task.Task`.
         :returns: Tasks stored on the cluster owned by the user.
 
@@ -451,7 +480,10 @@ class Connection(object):
         """
 
         url = get_url('tasks summaries') if summary else get_url('tasks')
-        if tags:
+        if tags_intersect:
+            tag_filter = all_tag_filter(tags_intersect)
+            response = self._post(get_url('tasks search'), tag_filter)
+        elif tags:
             response = self._get(url, params={'tag': tags})
         else:
             response = self._get(url)
