@@ -56,9 +56,12 @@ class Job(object):
         self._shortname = shortname
         self._pool_uuid = None
         if pool is not None:
-            self._pool_uuid = pool.uuid
+            if isinstance(pool, str):
+                self._pool_uuid = pool
+            else:
+                self._pool_uuid = pool.uuid
         self._state = ""
-        self._uuid = ""
+        self._uuid = None
         self._creation_date = datetime.datetime.now()
         self._use_dependencies = use_dependencies
         self._max_wall_time = None
@@ -67,6 +70,8 @@ class Job(object):
         self._last_auto_update_state = self._auto_update
 
         self._last_cache = time.time()
+        self._completion_time_to_live = "00:00:00"
+        self._auto_delete = False
 
     @property
     def auto_update(self):
@@ -138,11 +143,9 @@ class Job(object):
 
     @property
     def use_dependencies(self):
-        """:type: :class:`~qarnot.job.Job`
-        :getter: Returns this task's job
-        :setter: Sets this task's job
-
-        The job to submit the task in.
+        """:type: :class:`bool`
+        :getter: task's job can have dependencies
+        :setter: Set if there is task's job dependencies
 
         Can be set until :meth:`submit` is called.
         """
@@ -154,7 +157,7 @@ class Job(object):
         if self._uuid is not None:
             raise AttributeError("can't set attribute on a submitted job")
         else:
-            self._use_dependencies = value.uuid
+            self._use_dependencies = value
 
     @property
     def uuid(self):
@@ -224,6 +227,8 @@ class Job(object):
         :setter: Sets this job's maximum wall time
 
         The job's maximum wall time.
+        It is a time span string.
+        Format example: 'd.hh:mm:ss' or 'hh:mm:ss'
 
         Can be set until job is submitted.
         """
@@ -231,11 +236,15 @@ class Job(object):
 
     @max_wall_time.setter
     def max_wall_time(self, value):
-        """Setter for maximum wall time."""
+        """Setter for maximum wall time. In time span format example : 'd.hh:mm:ss' or 'hh:mm:ss' """
         if self._uuid is not None:
             raise AttributeError("can't set attribute on a submitted job")
-        else:
+        elif isinstance(value, str):
             self._max_wall_time = value
+        elif isinstance(value, datetime.timedelta):
+            self._max_wall_time = _util.convert_timedelta_to_timespan_string(value)
+        else:
+            raise TypeError("Maximum wall time must be a time span format string (example: 'd.hh:mm:ss' or 'hh:mm:ss')")
 
     @property
     def pool(self):
@@ -297,6 +306,9 @@ class Job(object):
         job._creation_date = payload["creationDate"]
         job._last_modified = payload["lastModified"]
         job._max_wall_time = _util.parse_timedelta(payload["maxWallTime"])
+
+        job._auto_delete = payload["autoDeleteOnCompletion"]
+        job._completion_time_to_live = payload["completionTimeToLive"]
         return job
 
     def _to_json(self):
@@ -307,7 +319,9 @@ class Job(object):
             'shortname': self._shortname,
             'state': self._state,
             'useDependencies': self._use_dependencies,
-            'maxWallTime': self._max_wall_time
+            'maxWallTime': self._max_wall_time,
+            'autoDeleteOnCompletion': self._auto_delete,
+            'completionTimeToLive': self._completion_time_to_live
         }
 
         return json_job
@@ -413,6 +427,54 @@ class Job(object):
         self._state = JobState.Deleting
         self._uuid = None
 
+    @property
+    def auto_delete(self):
+        """Autodelete this job if it is finished and your max number of job is reach
+
+        Can be set until :meth:`submit` is called.
+
+        :type: :class:`bool`
+        :getter: Returns is this job must autodelete
+        :setter: Sets this job's autodelete
+        :default_value: "False"
+
+        :raises AttributeError: if you try to reset the auto_delete after the job is submit
+        """
+        return self._auto_delete
+
+    @auto_delete.setter
+    def auto_delete(self, value):
+        """Setter for auto_delete, this can only be set before job's submission
+        """
+        if self.uuid is not None:
+            raise AttributeError("can't set attribute on a launched job")
+        self._auto_delete = value
+
+    @property
+    def completion_ttl(self):
+        """The job will be auto delete `completion_ttl` after it is finished
+
+        Can be set until :meth:`submit` is called.
+
+        :getter:  Returns this job's completed time to live.
+        :type: :class:`str`
+        :setter: Sets this job's this job's completed time to live.
+        :type: :class:`str` or :class:`datetime.timedelta`
+        :default_value: ""
+
+        :raises AttributeError: if you try to set it after the job is submitted
+
+        The `completion_ttl` must be a timedelta or a time span format string (example: 'd.hh:mm:ss' or 'hh:mm:ss')
+        """
+        return self._completion_time_to_live
+
+    @completion_ttl.setter
+    def completion_ttl(self, value):
+        """Setter for completion_ttl, this can only be set before job's submission"""
+        if self._uuid is not None:
+            raise AttributeError("can't set attribute on a submitted job")
+        self._completion_time_to_live = _util.parse_to_timespan_string(value)
+
     def __repr__(self):
         return '{0} - {1} - {2} - Pool : {3} - {4} - UseDependencies : {5} '\
             .format(self.name,
