@@ -19,6 +19,7 @@ import warnings
 from . import raise_on_error, get_url, _util
 from .bucket import Bucket
 from .status import Status
+from .error import Error
 from .exceptions import MissingPoolException, MaxPoolException, NotEnoughCreditsException, \
     BucketStorageUnavailableException, MissingBucketException
 
@@ -53,7 +54,7 @@ class Pool(object):
         self._state = 'UnSubmitted'  # RO property same for below
         self._profile = profile
         self._connection = connection
-        self.constants = {}
+        self._constants = {}
         """
          :type: dict(str, str)
 
@@ -63,7 +64,7 @@ class Pool(object):
         .. note:: See available constants for a specific profile
               with :meth:`qarnot.connection.Connection.retrieve_profile`.
         """
-        self.constraints = {}
+        self._constraints = {}
         self._auto_update = True
         self._last_auto_update_state = self._auto_update
         self._update_cache_time = 5
@@ -73,6 +74,7 @@ class Pool(object):
         self._resource_object_ids = []
         self._resource_objects = []
         self._tags = []
+        self._errors = None
         self._creation_date = None
         self._uuid = None
         self._max_objects_exceptions_class = MaxPoolException
@@ -86,6 +88,9 @@ class Pool(object):
         self._elastic_resize_factor = 1
         self._elastic_minimum_idle_time = 0
 
+        self._completion_time_to_live = "00:00:00"
+        self._auto_delete = False
+
     @classmethod
     def _retrieve(cls, connection, uuid):
         """Retrieve a submitted pool given its uuid.
@@ -129,6 +134,10 @@ class Pool(object):
         self._shortname = json_pool.get('shortname')
         self._profile = json_pool['profile']
         self._instancecount = json_pool['instanceCount']
+        if 'errors' in json_pool:
+            self._errors = [Error(d) for d in json_pool['errors']]
+        else:
+            self._errors = []
 
         if 'resourceBuckets' in json_pool and json_pool['resourceBuckets'] is not None:
             self._resource_object_ids = json_pool['resourceBuckets']
@@ -139,11 +148,17 @@ class Pool(object):
 
         if 'constants' in json_pool:
             for constant in json_pool['constants']:
-                self.constants[constant.get('key')] = constant.get('value')
+                self._constants[constant.get('key')] = constant.get('value')
         self._uuid = json_pool['uuid']
         self._state = json_pool['state']
         self._tags = json_pool.get('tags', None)
 
+        if 'autoDeleteOnCompletion' in json_pool:
+            self._auto_delete = json_pool["autoDeleteOnCompletion"]
+
+        if 'completionTimeToLive' in json_pool:
+            self._completion_time_to_live = json_pool["completionTimeToLive"]
+
         if 'elasticProperty' in json_pool:
             elasticProperty = json_pool["elasticProperty"]
             self._is_elastic = elasticProperty["isElastic"]
@@ -158,11 +173,11 @@ class Pool(object):
         """Get a dict ready to be json packed from this pool."""
         const_list = [
             {'key': key, 'value': value}
-            for key, value in self.constants.items()
+            for key, value in self._constants.items()
         ]
         constr_list = [
             {'key': key, 'value': value}
-            for key, value in self.constraints.items()
+            for key, value in self._constraints.items()
         ]
 
         elastic_dict = {
@@ -188,8 +203,12 @@ class Pool(object):
         if self._shortname is not None:
             json_pool['shortname'] = self._shortname
 
+        self._resource_object_ids = [x.uuid for x in self._resource_objects]
         json_pool['resourceBuckets'] = self._resource_object_ids
 
+        json_pool['autoDeleteOnCompletion'] = self._auto_delete
+        json_pool['completionTimeToLive'] = self._completion_time_to_live
+
         return json_pool
 
     def submit(self):
@@ -249,15 +268,38 @@ class Pool(object):
         self._is_summary = False
         self._last_cache = time.time()
 
-    def apply_elastic_settings(self):
-        response = self._connection._put('pool update', json=self._to_json())
+    def commit(self):
+        """Replicate local changes on the current object instance to the REST API
+
+        :raises qarnot.exceptions.QarnotGenericException: API general error, see message for details
+        :raises qarnot.exceptions.UnauthorizedException: invalid credentials
+
+        This function need to be call to apply the local elastic pool setting modifications.
+        .. note:: When updating buckets' properties, auto update will be disabled until commit is called.
+        """
+        response = self._connection._put(get_url('pool update', uuid=self._uuid), json=self._to_json())
+        self._auto_update = self._last_auto_update_state
+
         if response.status_code == 404:
             raise MissingPoolException(response.json()['message'])
-
         raise_on_error(response)
-        self.update()
 
     def setup_elastic(self, minimum_total_slots=0, maximum_total_slots=1, minimum_idle_slots=0, minimum_idle_time_seconds=0, resize_factor=1, resize_period=90):
+        """Setup the pool elastic properties
+
+        :param int minimum_total_slots: Minimum slot number for the pool in elastic mode.
+                Defaults to 0.
+        :param int maximum_total_slots: Maximum slot number for the pool in elastic mode.
+                Defaults to 1.
+        :param int minimum_idle_slots: Minimum idling slot number.
+                Defaults to 0.
+        :param int minimum_idle_time_seconds: Wait time in seconds before closing an unused slot if the number of unused slots are upper than the minimum_idle_slots.
+                Defaults to 0.
+        :param float resize_factor: Growing factor of the pool. It must be a number between 0 and 1.
+                Defaults to 1.
+        :param int resize_period: Refresh rate of resizing the pool in elastic mode.
+                Defaults to 90.
+        """
         self._is_elastic = True
         self._elastic_maximum_slots = maximum_total_slots
         self._elastic_minimum_slots = minimum_total_slots
@@ -422,6 +464,7 @@ class Pool(object):
 
     @tags.setter
     def tags(self, value):
+        """Setter for tags"""
         self._tags = value
         self._auto_update = False
 
@@ -464,6 +507,19 @@ class Pool(object):
             raise AttributeError("can't set attribute on a launched pool")
         self._instancecount = value
 
+    @property
+    def errors(self):
+        """:type: list(:class:`str`)
+        :getter: Returns this pool's error list
+
+        Error reason if any, empty string if none
+        """
+        self._update_if_summary()
+        if self._auto_update:
+            self.update()
+
+        return self._errors
+
     @property
     def creation_date(self):
         """:type: :class:`str`
@@ -536,6 +592,12 @@ class Pool(object):
 
     @property
     def is_elastic(self):
+        """:type: :class:`bool`
+        :getter: Returns this pool's is_elastic
+        :setter: Sets this pool's is_elastic
+
+        Define if you use a static or an elastic pool.
+        """
         return self._is_elastic
 
     @is_elastic.setter
@@ -544,42 +606,87 @@ class Pool(object):
 
     @property
     def elastic_minimum_slots(self):
+        """:type: :class:`int`
+        :getter: Returns this pool's elastic_minimum_slots
+        :setter: Sets this pool's elastic_minimum_slots
+
+        The minimum slot number of the elastic pool.
+        Define the minimum number of pool instances stay open during the pool execution.
+        """
         return self._elastic_minimum_slots
 
     @elastic_minimum_slots.setter
     def elastic_minimum_slots(self, value):
+        """Setter for elastic_minimum_slots"""
         self._elastic_minimum_slots = value
 
     @property
     def elastic_maximum_slots(self):
+        """:type: :class:`int`
+        :getter: Returns this pool's elastic_maximum_slots
+        :setter: Sets this pool's elastic_maximum_slots
+
+        The maximum slot number of the elastic pool.
+        Define the maximum number of pool instances opened during the pool execution.
+        """
         return self._elastic_maximum_slots
 
     @elastic_maximum_slots.setter
     def elastic_maximum_slots(self, value):
+        """Setter for elastic_maximum_slots"""
         self._elastic_maximum_slots = value
 
     @property
     def elastic_minimum_idle_slots(self):
+        """:type: :class:`int`
+        :getter: Returns this pool's elastic_minimum_idle_slots
+        :setter: Sets this pool's elastic_minimum_idle_slots
+
+        The minimum idle number of the elastic pool.
+        Define the minimum number of the idle pool instances stay opened during the pool execution.
+        It should be lower to elastic_minimum_slots to be usefull
+        """
         return self._elastic_minimum_idle_slots
 
     @elastic_minimum_idle_slots.setter
     def elastic_minimum_idle_slots(self, value):
+        """Setter for elastic_minimum_idle_slots"""
         self._elastic_minimum_idle_slots = value
 
     @property
     def elastic_minimum_idle_time(self):
+        """:type: :class:`int`
+        :getter: Returns this pool's elastic_minimum_idle_time
+        :setter: Sets this pool's elastic_minimum_idle_time
+
+        Wait time in seconds before closing an unused slot if the number of unused slots are upper than the minimum_idle_slots.
+        """
         return self._elastic_minimum_idle_time
 
     @elastic_minimum_idle_time.setter
     def elastic_minimum_idle_time(self, value):
+        """Setter for elastic_minimum_idle_time"""
         self._elastic_minimum_idle_time = value
 
     @property
     def elastic_resize_factor(self):
+        """:type: :class:`float`
+        :getter: Returns this pool's elastic_resize_factor
+        :setter: Sets this pool's elastic_resize_factor
+
+        The resize factor of the pool.
+        It represent the resize factor of the slots.
+        It's a decimal number upper than 0 and and equal or lower the 1
+        """
         return self._elastic_resize_factor
 
     @elastic_resize_factor.setter
     def elastic_resize_factor(self, value):
+        """Setter for elastic_resize_factor
+
+        :raises Exception: resize factor must be > 0
+        :raises Exception: resize factor must be <= 1
+        """
         if value <= 0:
             raise Exception("resize factor must be > 0")
         elif value > 1:
@@ -588,16 +695,128 @@ class Pool(object):
 
     @property
     def elastic_resize_period(self):
+        """:type: :class:`int`
+        :getter: Returns this pool's elastic_resize_period
+        :setter: Sets this pool's elastic_resize_period
+
+        The resize period of the elastic pool in second.
+        This is the refresh rate of resizing the elastic pool.
+        """
         return self._elastic_resize_period
 
     @elastic_resize_period.setter
     def elastic_resize_period(self, value):
+        """Setter for elastic_resize_period"""
         self._elastic_resize_period = value
 
+    @property
+    def constants(self):
+        """:type: dictionary{:class:`str` : :class:`str`}
+        :getter: Returns this pool's constants dictionary.
+        :setter: set the pool's constants dictionary.
+
+        Update the constants if needed
+        Constants are the parametrazer of the profils.
+        Use them to adjust your profile parametter.
+        """
+        self._update_if_summary()
+        if self._auto_update:
+            self.update()
+
+        return self._constants
+
+    @constants.setter
+    def constants(self, value):
+        """Setter for constants
+        """
+        self._update_if_summary()
+        if self._auto_update:
+            self.update()
+
+        self._constants = value
+
+    @property
+    def constraints(self):
+        """:type: dictionary{:class:`str` : :class:`str`}
+        :getter: Returns this pool's constraints dictionary.
+        :setter: set the pool's constraints dictionary.
+
+        Update the constraints if needed
+        advance usage
+        """
+        self._update_if_summary()
+        if self._auto_update:
+            self.update()
+
+        return self._constraints
+
+    @constraints.setter
+    def constraints(self, value):
+        """Setter for constraints
+        """
+        self._update_if_summary()
+        if self._auto_update:
+            self.update()
+
+        self._constraints = value
+
+    @property
+    def auto_delete(self):
+        """Autodelete this pool if it is finished and your max number of pool is reach
+
+        Can be set until :meth:`submit` is called.
+
+        :type: :class:`bool`
+        :getter: Returns is this pool must autodelete
+        :setter: Sets this pool's autodelete
+        :default_value: "False"
+
+        :raises AttributeError: if you try to reset the auto_delete after the pool is submit
+        """
+        self._update_if_summary()
+        return self._auto_delete
+
+    @auto_delete.setter
+    def auto_delete(self, value):
+        """Setter for auto_delete, this can only be set before pool's submission
+        """
+        self._update_if_summary()
+        if self.uuid is not None:
+            raise AttributeError("can't set attribute on a launched pool")
+        self._auto_delete = value
+
+    @property
+    def completion_ttl(self):
+        """The pool will be auto delete `completion_ttl` after it is finished
+
+        Can be set until :meth:`submit` is called.
+
+        :getter:  Returns this pool's completed time to live.
+        :type: :class:`str`
+        :setter: Sets this pool's this pool's completed time to live.
+        :type: :class:`str` or :class:`datetime.timedelta`
+        :default_value: ""
+
+        :raises AttributeError: if you try to set it after the pool is submitted
+
+        The `completion_ttl` must be a timedelta or a time span format string (example: 'd.hh:mm:ss' or 'hh:mm:ss')
+        """
+        self._update_if_summary()
+        return self._completion_time_to_live
+
+    @completion_ttl.setter
+    def completion_ttl(self, value):
+        """Setter for completion_ttl, this can only be set before pool's submission"""
+        self._update_if_summary()
+        if self._uuid is not None:
+            raise AttributeError("can't set attribute on a submitted job")
+        self._completion_time_to_live = _util.parse_to_timespan_string(value)
+
     def __repr__(self):
         return '{0} - {1} - {2} - {3} - {5} - InstanceCount : {4} - Resources : {6} '\
             'Tag {7} - IsElastic {8} - ElasticMin {9} - ElasticMax {10} - ElasticMinIdle {11} -'\
-            ' ElasticResizePeriod {12} - ElasticResizeFactor {13} - ElasticMinIdleTimeSeconds {14}'\
+            ' ElasticResizePeriod {12} - ElasticResizeFactor {13} - ElasticMinIdleTimeSeconds {14} - '\
+            'Errors {15}'\
             .format(self.name,
                     self.shortname,
                     self._uuid,
@@ -612,4 +831,5 @@ class Pool(object):
                     self._elastic_minimum_idle_slots,
                     self._elastic_resize_period,
                     self._elastic_resize_factor,
-                    self._elastic_minimum_idle_time)
+                    self._elastic_minimum_idle_time,
+                    self._errors)
