@@ -24,6 +24,7 @@ from . import get_url, raise_on_error, _util
 from .status import Status
 from .bucket import Bucket
 from .pool import Pool
+from .error import Error
 from .exceptions import MissingTaskException, MaxTaskException, NotEnoughCreditsException, \
     MissingBucketException, BucketStorageUnavailableException
 
@@ -89,7 +90,7 @@ class Task(object):
 
         self._result_object = None
         self._connection = connection
-        self.constants = {}
+        self._constants = {}
         """
          :type: dict(str,str)
 
@@ -107,10 +108,9 @@ class Task(object):
         self._update_cache_time = 5
 
         self._last_cache = time.time()
-        self.constraints = {}
+        self._constraints = {}
         self._state = 'UnSubmitted'  # RO property same for below
         self._uuid = None
-        self._shortname = None
         self._snapshots = False
         self._dirty = False
         self._rescount = -1
@@ -126,6 +126,8 @@ class Task(object):
         self._resource_object_ids = []
         self._result_object_id = None
         self._is_summary = False
+        self._completion_time_to_live = "00:00:00"
+        self._auto_delete = False
 
     @classmethod
     def _retrieve(cls, connection, uuid):
@@ -389,7 +391,7 @@ class Task(object):
 
         if 'constants' in json_task:
             for constant in json_task['constants']:
-                self.constants[constant.get('key')] = constant.get('value')
+                self._constants[constant.get('key')] = constant.get('value')
 
         self._uuid = json_task['uuid']
         self._state = json_task['state']
@@ -413,6 +415,11 @@ class Task(object):
         else:
             self._completed_instances = []
 
+        if 'autoDeleteOnCompletion' in json_task:
+            self._auto_delete = json_task["autoDeleteOnCompletion"]
+        if 'completionTimeToLive' in json_task:
+            self._completion_time_to_live = json_task["completionTimeToLive"]
+
     @classmethod
     def from_json(cls, connection, json_task, is_summary=False):
         """Create a Task object from a json task.
@@ -826,6 +833,7 @@ class Task(object):
 
     @tags.setter
     def tags(self, value):
+        """setter for tags"""
         self._tags = value
         self._auto_update = False
 
@@ -1068,6 +1076,57 @@ class Task(object):
 
         return self._errors
 
+    @property
+    def constants(self):
+        """:type: dictionary{:class:`str` : :class:`str`}
+        :getter: Returns this task's constants dictionary.
+        :setter: set the task's constants dictionary.
+
+        Update the constants if needed
+        Constants are the parametrazer of the profils.
+        Use them to adjust your profile parametter.
+        """
+        self._update_if_summary()
+        if self._auto_update:
+            self.update()
+
+        return self._constants
+
+    @constants.setter
+    def constants(self, value):
+        """Setter for constants
+        """
+        self._update_if_summary()
+        if self._auto_update:
+            self.update()
+
+        self._constants = value
+
+    @property
+    def constraints(self):
+        """:type: dictionary{:class:`str` : :class:`str`}
+        :getter: Returns this task's constraints dictionary.
+        :setter: set the task's constraints dictionary.
+
+        Update the constraints if needed
+        advance usage
+        """
+        self._update_if_summary()
+        if self._auto_update:
+            self.update()
+
+        return self._constraints
+
+    @constraints.setter
+    def constraints(self, value):
+        """Setter for constraints
+        """
+        self._update_if_summary()
+        if self._auto_update:
+            self.update()
+
+        self._constraints = value
+
     @property
     def auto_update(self):
         """:type: :class:`bool`
@@ -1106,20 +1165,76 @@ class Task(object):
         self._update_cache_time = value
 
     def set_task_dependencies_from_uuids(self, uuids):
+        """Setter for the task dependencies using uuid
+        """
         self._dependentOn += uuids
 
     def set_task_dependencies_from_tasks(self, tasks):
+        """Setter for the task dependencies using tasks
+        """
         self._dependentOn += [task._uuid for task in tasks]
 
+    @property
+    def auto_delete(self):
+        """Autodelete this Task if it is finished and your max number of task is reach
+
+        Can be set until :meth:`run` or :meth:`submit` is called.
+
+        :type: :class:`bool`
+        :getter: Returns is this task must autodelete
+        :setter: Sets this task's autodelete
+        :default_value: "False"
+
+        :raises AttributeError: if you try to reset the auto_delete after the task is submit
+        """
+        self._update_if_summary()
+        return self._auto_delete
+
+    @auto_delete.setter
+    def auto_delete(self, value):
+        """Setter for auto_delete, this can only be set before tasks submission
+        """
+        self._update_if_summary()
+        if self.uuid is not None:
+            raise AttributeError("can't set attribute on a launched task")
+        self._auto_delete = value
+
+    @property
+    def completion_ttl(self):
+        """The task will be auto delete `completion_ttl` after it is finished
+
+        Can be set until :meth:`run` or :meth:`submit` is called.
+
+        :getter:  Returns this task's completed time to live.
+        :type: :class:`str`
+        :setter: Sets this task's this task's completed time to live.
+        :type: :class:`str` or :class:`datetime.timedelta`
+        :default_value: ""
+
+        :raises AttributeError: if you try to set it after the task is submitted
+
+        The `completion_ttl` must be a timedelta or a time span format string (example: 'd.hh:mm:ss' or 'hh:mm:ss')
+        """
+        self._update_if_summary()
+        return self._completion_time_to_live
+
+    @completion_ttl.setter
+    def completion_ttl(self, value):
+        """Setter for completion_ttl, this can only be set before tasks submission"""
+        self._update_if_summary()
+        if self._uuid is not None:
+            raise AttributeError("can't set attribute on a submitted job")
+        self._completion_time_to_live = _util.parse_to_timespan_string(value)
+
     def _to_json(self):
         """Get a dict ready to be json packed from this task."""
         const_list = [
             {'key': key, 'value': value}
-            for key, value in self.constants.items()
+            for key, value in self._constants.items()
         ]
         constr_list = [
             {'key': key, 'value': value}
-            for key, value in self.constraints.items()
+            for key, value in self._constraints.items()
         ]
 
         json_task = {
@@ -1157,6 +1272,9 @@ class Task(object):
             json_task['resultsWhitelist'] = self._results_whitelist
         if self._results_blacklist is not None:
             json_task['resultsBlacklist'] = self._results_blacklist
+
+        json_task['autoDeleteOnCompletion'] = self._auto_delete
+        json_task['completionTimeToLive'] = self._completion_time_to_live
         return json_task
 
     def _update_if_summary(self):
@@ -1188,34 +1306,6 @@ class Task(object):
         return False
 
 
-class Error(object):
-    """Task error
-
-    .. note:: Read-only class
-    """
-    def __init__(self, json):
-        self.code = json['code']
-        """:type: :class:`str`
-
-        Error code."""
-
-        self.message = json['message']
-        """:type: :class:`str`
-
-        Error message."""
-
-        self.debug = json['debug']
-        """:type: :class:`str`
-
-        Optional extra debug information"""
-
-    def __repr__(self):
-        if sys.version_info > (3, 0):
-            return ', '.join("{0}={1}".format(key, val) for (key, val) in self.__dict__.items())
-        else:
-            return ', '.join("{0}={1}".format(key, val) for (key, val) in self.__dict__.iteritems())  # pylint: disable=no-member
-
-
 class CompletedInstance(object):
     """Completed Instance Information
 
